@@ -103,7 +103,7 @@ def request(scope, *, requires_login=False):
             origin = get_origin(type_)
             args = get_args(type_)
             if origin is Union and is_base_model_type(args[0]):
-                instantiate[name] = args[0]
+                instantiate[name] = type_
 
         arg_names = list(inspect.signature(function).parameters)
 
@@ -149,6 +149,12 @@ def request(scope, *, requires_login=False):
             for i, (arg_name, arg) in enumerate(zip(arg_names, args)):
                 if arg_name in instantiate:
                     type_ = instantiate[arg_name]
+                    # allow users to pass None for optional args. Without this
+                    # we would try to instantiate types like `GameMode(None)`
+                    # which would error.
+                    if is_optional(type_) and arg is None:
+                        continue
+                    type_ = get_args(type_)[0]
                     args[i] = type_(arg)
                 id_ = id_from_id_type(arg_name, arg)
                 if id_:
@@ -157,6 +163,9 @@ def request(scope, *, requires_login=False):
             for arg_name, arg in kwargs.items():
                 if arg_name in instantiate:
                     type_ = instantiate[arg_name]
+                    if is_optional(type_) and arg is None:
+                        continue
+                    type_ = get_args(type_)[0]
                     kwargs[arg_name] = type_(arg)
                 id_ = id_from_id_type(arg_name, arg)
                 if id_:
@@ -444,13 +453,17 @@ class OssapiV2:
         self.log.info(f"made {method} request to {r.request.url}")
         json_ = r.json()
         self.log.debug(f"received json: \n{json.dumps(json_, indent=4)}")
+        self._check_response(json_, url)
+
+        return self._instantiate_type(type_, json_)
+
+    def _check_response(self, json_, url):
         # TODO this should just be ``if "error" in json``, but for some reason
         # ``self.search_beatmaps`` always returns an error in the response...
         # open an issue on osu-web?
         if len(json_) == 1 and "error" in json_:
             raise ValueError(f"api returned an error of `{json_['error']}` for "
                 f"a request to {unquote(url)}")
-        return self._instantiate_type(type_, json_)
 
     def _get(self, type_, url, params={}):
         return self._request(type_, "GET", url, params=params)
@@ -1176,10 +1189,27 @@ class OssapiV2:
     @request(Scope.PUBLIC, requires_login=True)
     def download_score(self,
         mode: GameModeT,
-        score_id: int
+        score_id: int,
+        *,
+        raw: bool = False
     ) -> Replay:
-        r = self.session.get(f"{self.BASE_URL}/scores/{mode.value}/"
-            f"{score_id}/download")
+        url = f"{self.BASE_URL}/scores/{mode.value}/{score_id}/download"
+        r = self.session.get(url)
+
+        # if the response above succeeded, it will return a raw string
+        # instead of json. If it didn't succeed, it will return json with an
+        # error.
+        # So always try parsing as json to check if there's an error. If parsin
+        # fails, just assume the request succeeded and move on.
+        try:
+            json_ = r.json()
+            self._check_response(json_, url)
+        except json.JSONDecodeError:
+            pass
+
+        if raw:
+            return r.content
+
         replay = osrparse.Replay.from_string(r.content)
         return Replay(replay, self)
 
